/***********************************************************************/
/*                                                                     */
/*   svm_classify.c                                                    */
/*                                                                     */
/*   Classification module of Support Vector Machine.                  */
/*                                                                     */
/*   Copyright: Thorsten Joachims                                      */
/*   Date: 11.11.97                                                    */
/*                                                                     */
/***********************************************************************/

# include <stdio.h>
# include <ctype.h>
# include <signal.h>
# include <math.h>
# include <string.h>
# include <stdlib.h>
# include "svm.h"

long verbosity;              /* verbosity level (0-3) */

char docfile[200];
char modelfile[200];
char predictionsfile[200];

main (argc, argv)
int argc;
char *argv[];
{
  WORD **docs;   /* test examples */
  long max_docs,max_words_doc,i,j,k,lld,llsv;
  long max_sv,max_words_sv,totwords,totdoc,*label;
  long correct=0,incorrect=0,no_accuracy=0;
  long res_a=0,res_b=0,res_c=0,res_d=0;
  double dist;
  FILE *predfl;
  KERNEL_PARM kernel_parm;
  MODEL model;

  read_input_parameters(argc,argv,docfile,modelfile,predictionsfile,
			&verbosity);

  nol_ll(docfile,&max_docs,&max_words_doc,&lld); /* scan size of input file */
  max_words_doc+=2;
  lld+=2;
  nol_ll(modelfile,&max_sv,&max_words_sv,&llsv); /* scan size of model file */
  max_words_sv+=2;
  llsv+=2;

  docs = (WORD **)malloc(sizeof(WORD *)*max_docs);
  label = (long *)malloc(sizeof(long)*max_docs);
  model.supvec = (WORD **)malloc(sizeof(double *)*max_sv);
  model.alpha = (double *)malloc(sizeof(double)*max_sv);

  read_documents(docfile,docs,label,max_words_doc,lld,&totwords,&totdoc);
  read_model(modelfile,&model,max_words_sv,llsv,&kernel_parm);
  
  if(verbosity>=2) {
    printf("Classifying test examples.."); fflush(stdout);
  }
  if ((predfl = fopen (predictionsfile, "w")) == NULL)
  { perror (predictionsfile); exit (1); }

  for(i=0;i<totdoc;i++) {
    dist=classify_example(&model,docs[i],&kernel_parm);
    if(dist>0) {
      fprintf(predfl,"%.8g:+1 %.8g:-1\n",dist,-dist);
      if(label[i]==1) correct++; else incorrect++;
      if(label[i]==1) res_a++; else res_b++;
    }
    else {
      fprintf(predfl,"%.8g:-1 %.8g:+1\n",-dist,dist);
      if(label[i]==-1) correct++; else incorrect++;
      if(label[i]==1) res_c++; else res_d++;
    }
    if(abs(label[i]) != 1) no_accuracy=1;
    if(verbosity>=2) {
      if(i % 100 == 0) {
	printf("%ld..",i); fflush(stdout);
      }
    }
  }  
  free(docs);
  free(label);
  free(model.supvec);
  free(model.alpha);
  if(verbosity>=2) {
    printf("done\n");
  }
  if((!no_accuracy) && (verbosity>=1)) {
    printf("Accuracy on test set: %.2lf%% (%ld correct, %ld incorrect, %ld total)\n",(double)correct*100/totdoc,correct,incorrect,totdoc);
    printf("Precision/recall on test set: %.2lf%%/%.2lf%%\n",(double)res_a*100/(res_a+res_b),(double)res_a*100/(res_a+res_c));
  }
}

read_model(modelfile,model,max_words,ll,kernel_parm)
char *modelfile;
MODEL *model;
long max_words,ll;
KERNEL_PARM *kernel_parm;
{
  FILE *modelfl;
  long j,i;
  char *line;
  WORD *doc;
  register long wpos;
  long wnum,pos;
  double weight;

  if(verbosity>=1) {
    printf("Reading model..."); fflush(stdout);
  }
  doc = (WORD *)malloc(sizeof(WORD)*(max_words+10));
  line = (char *)malloc(sizeof(char)*ll);

  if ((modelfl = fopen (modelfile, "r")) == NULL)
  { perror (modelfile); exit (1); }
  fscanf(modelfl,"%ld\n", &kernel_parm->kernel_type);  
  fscanf(modelfl,"%ld\n", &kernel_parm->poly_degree);
  fscanf(modelfl,"%lf\n", &kernel_parm->rbf_gamma);
  fscanf(modelfl,"%lf\n", &kernel_parm->sigmoid_gamma);
  fscanf(modelfl,"%lf\n", &kernel_parm->sigmoid_c);

  fscanf(modelfl,"%ld\n", &model->sv_num);
  fscanf(modelfl,"%lf\n", &model->b);
  for(i=1;i<model->sv_num;i++) {
    fscanf(modelfl,"%[^\n]\n", line);
    pos=0;
    wpos=0;
    sscanf(line,"%lf",&model->alpha[i]);
    while(line[++pos]>32);
    while((sscanf(line+pos,"%ld:%lf",&wnum,&weight) != EOF) 
	  && (wpos<max_words)) {
      while(line[++pos]>32);
      doc[wpos].wnum=wnum;
      doc[wpos].weight=weight; 
      wpos++;
    }
    model->supvec[i] = (WORD *)malloc(sizeof(WORD)*(wpos+1));
    for(j=0;j<wpos;j++) (model->supvec[i])[j]=doc[j];
    (model->supvec[i])[wpos].wnum=0;
  }
  fclose(modelfl);
  free(line);
  free(doc);
  if(verbosity>=1) {
    fprintf(stdout, "OK. (%d support vectors read)\n", model->sv_num-1);
  }
}

read_input_parameters(argc,argv,docfile,modelfile,predictionsfile,verbosity)
int argc;
char *argv[],*docfile,*modelfile,*predictionsfile;
long *verbosity;
{
  long i;
  
  /* set default */
  strcpy (modelfile, "svm_model");
  strcpy (predictionsfile, "svm_predictions"); 
  (*verbosity)=2;

  for(i=1;(i<argc) && ((argv[i])[0] == '-');i++) {
    switch ((argv[i])[1]) 
      { 
      case 'h': print_help(); exit(0);
      case 'v': i++; (*verbosity)=atol(argv[i]); break;
      default: printf("\nUnrecognized option %s!\n\n",argv[i]);
	       print_help();
	       exit(0);
      }
  }
  if((i+1)>=argc) {
    printf("\nNot enough input parameters!\n\n");
    print_help();
    exit(0);
  }
  strcpy (docfile, argv[i]);
  strcpy (modelfile, argv[i+1]);
  if((i+2)<argc) {
    strcpy (predictionsfile, argv[i+2]);
  }
}

print_help()
{
  printf("\nSVM-light V0.9: Support Vector Machine, classification module     11.11.97\n");
  copyright_notice();
  printf("   usage: svm_classify [options] example_file model_file output_file\n\n");
  printf("options: -h         -> this help\n");
  printf("         -v [0..3]  -> verbosity level (default 2)\n\n");
}


